import time
from urllib.parse import urlparse, urlunparse
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException
from linkedin_cat.helper import extract_and_decode_username


from linkedin_cat.helper import (
    scroll_and_load,
    get_object,
    get_objects,
    extract_element_text,
    extract_element_attribute,
    extract_many_element_text,
    save_to_json,
)

def extract_company_people_links(driver, company_url):
    # Go to the company URL
    driver.get(company_url)

    # Get all the profile cards with their links
    profile_links_xpath = "//ul[@class='display-flex list-style-none flex-wrap']/li//a[contains(@class, 'app-aware-link')]"
    profile_links = get_objects(driver, By.XPATH, profile_links_xpath)

    # Extract the href (profile link) from each card
    links = []
    for profile in profile_links:
        link = extract_element_attribute(profile, By.XPATH, ".", "href")
        links.append(link)

    return links


def extract_intro(driver):
    try:
        intro_data = {}

        # XPath for the intro section
        intro_xpath = "//div[contains(@class, 'mt2 relative')]"
        intro = get_object(driver, By.XPATH, intro_xpath)
        if intro:
            # Extract name using XPath
            name_xpath = ".//h1[contains(@class, 'text-heading-xlarge')]"
            name = extract_element_text(intro, By.XPATH, name_xpath)
            intro_data["name"] = name

            # Extract pronouns (if available) using XPath
            pronouns_xpath = (
                ".//span[contains(@class, 'text-body-small v-align-middle')]"
            )
            try:
                pronouns = extract_element_text(intro, By.XPATH, pronouns_xpath)
                intro_data["pronouns"] = pronouns
            except:
                intro_data["pronouns"] = None

            # Extract company/workplace (works_at) using XPath
            works_at_xpath = ".//div[contains(@class, 'text-body-medium break-words')]"
            works_at = extract_element_text(intro, By.XPATH, works_at_xpath)
            intro_data["works_at"] = works_at
            # Extract location using XPath
            location_xpath = ".//span[contains(@class, 'text-body-small inline t-black--light break-words')]"
            location = extract_element_text(intro, By.XPATH, location_xpath)
            intro_data["location"] = location

            # Extract followers count using XPath
            followers_xpath = "//p[contains(@class, 'pvs-header__optional-link')]//span[contains(text(), 'followers')]"
            try:
                followers = extract_element_text(
                    driver, By.XPATH, followers_xpath
                ).split()[0]
                intro_data["followers"] = followers
            except:
                intro_data["followers"] = None

            # Extract connections count using XPath
            connections_xpath = "//li[@class='text-body-small']//span[@class='t-black--light']//span[@class='t-bold']"
            try:
                connections = extract_element_text(driver, By.XPATH, connections_xpath)
                intro_data["connections"] = connections
            except:
                intro_data["connections"] = None

        else:
            print("Intro section not found")
            intro_data = None

    except NoSuchElementException as e:
        print(f"Error extracting intro: Element not found - Not Available")
        intro_data = None
    except Exception as e:
        print(f"Error extracting intro: Not Available")
        intro_data = None

    return intro_data

def extract_about(driver):
    try:
        xpath_about = "//section[contains(@class, 'artdeco-card pv-profile-card')]//h2[span[text()='About']]/ancestor::section//div[contains(@class, 'full-width')]//span[@aria-hidden='true']"

        about_description = extract_element_text(
            driver, By.XPATH, xpath_about, timeout=2
        )

        if about_description:
            return {"about_description": about_description}
        else:
            return {"about_description": ""}

    except Exception as e:
        print(f"'About' section doesn't exist")
        return {"about_description": ""}

def extract_experience(driver):
    experience_data = []
    try:
        # Define the list to store all experience data
        scroll_and_load(driver)
        # Define XPath for extracting job experience information
        experience_elements = get_objects(
            driver,
            By.XPATH,
            "//div[@id='experience']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
            timeout=5
        )

        # Loop through the experiences and extract information
        for experience in experience_elements:
            # Initialize a dictionary to store the data for each experience
            experience_dict = {}

            try:
                # Check if the experience has multiple roles (nested list)
                try:
                    nested_experiences = get_objects(
                        experience,
                        By.XPATH,
                        ".//div[contains(@class, 'pvs-entity__sub-components')]/ul/li/div[contains(@data-view-name, 'profile-component-entity')]",
                    )
                except Exception as e:
                    nested_experiences = None

                if nested_experiences:
                    try:
                        company_name = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//div[contains(@class, 'flex-wrap')]",
                        ).split("\n")[0]
                    except Exception as e:
                        print(f"Error extracting company name: Not available")
                        company_name = None

                    # try:
                    #     total_time = extract_element_text(
                    #         experience,
                    #         By.XPATH,
                    #         ".//span[@class='t-14 t-normal']"
                    #     ).split("\n")[0]
                    #     experience_dict['total_time'] = total_time
                    # except Exception as e:
                    #     print(f"Error extracting total time: Not available")
                    #     experience_dict['total_time'] = None

                    try:
                        location = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//a[@data-field='experience_company_logo']//span[contains(@class, 't-black--light')]"
                        ).split("\n")[0]
                    except Exception as e:
                        print(f"Error extracting location: Not available")
                        location = None

                    for nested_exp in nested_experiences:
                        # Try to extract the job title
                        try:
                            job_title = extract_element_text(
                                nested_exp,
                                By.XPATH,
                                ".//div[@class='display-flex flex-wrap align-items-center full-height']"
                            ).split('\n')[0]
                        except Exception as e:
                            print(f"Error extracting job title: Not available")
                            job_title = None

                        # Try to extract the type
                        try:
                            type = extract_element_text(
                                nested_exp,
                                By.XPATH,
                                ".//span[@class='t-14 t-normal']"
                            ).split()[0]
                        except Exception as e:
                            print(f"Error extracting type: Not available")
                            type = None

                        # Try to extract the dates
                        try:
                            dates = extract_element_text(
                                nested_exp,
                                By.XPATH,
                                './/span[contains(@class, "t-black--light")]/span'
                            )
                        except Exception as e:
                            print(f"Error extracting dates: Not available")
                            dates = None

                        try:
                            description = extract_element_text(
                                nested_exp,
                                By.XPATH,
                                ".//li[contains(@class, 'pvs-list__item--with-top-padding')]//div[contains(@class, 'inline-show-more-text--is-collapsed')]",
                            )
                        except Exception as e:
                            print(f"Error extracting description: Not available")
                            description = None

                        experience_dict = {
                            'company_name': company_name,
                            'job_title': job_title,
                            'location': location,
                            'type': type,
                            'dates': dates,
                            'description': description
                        }
                        experience_data.append(experience_dict)
                else:
                    # Try to extract the job title
                    try:
                        company_type = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//span[@class='t-14 t-normal']"
                        )
                        company_type = company_type.split('\n')[0]
                        company_type = company_type.split('·')
                        company_name = company_type[0]
                        type = company_type[1]
                    except Exception as e:
                        print(f"Error extracting job title: Not available")
                        company_name = None
                        type = None

                    try:
                        job_title = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//div[contains(@class, 'flex-wrap')]",
                        )
                    except Exception as e:
                        print(f"Error extracting company name: Not available")
                        job_title = None

                    # Try to extract the dates
                    try:
                        dates = extract_element_text(
                            experience,
                            By.XPATH,
                            './/span[contains(@class, "t-black--light")]/span'
                        )
                    except Exception as e:
                        print(f"Error extracting dates: Not available")
                        dates = None

                    # Try to extract the location
                    try:
                        location = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//span[contains(@class, 't-black--light')]//span[not(contains(@class, 'pvs-entity__caption-wrapper')) and @aria-hidden='true']",
                        )
                    except Exception as e:
                        print(f"Error extracting location: Not available")
                        location = None

                    try:
                        description = extract_element_text(
                            experience,
                            By.XPATH,
                            ".//li[contains(@class, 'pvs-list__item--with-top-padding')]//div[contains(@class, 'inline-show-more-text--is-collapsed')]",
                        )
                    except Exception as e:
                        print(f"Error extracting description: Not available")
                        description = None

                    experience_dict = {
                        'company_name': company_name,
                        'job_title': job_title,
                        'location': location,
                        'type': type,
                        'dates': dates,
                        'description': description
                    }
                    experience_data.append(experience_dict)
            except Exception as e:
                print(f"Error extracting experience information: {e}")
    except Exception as e:
        print(f"Error extracting experience information: Not Available")

    return experience_data

def extract_education(driver):
    educations = []
    try:

        # Locate all education entries
        education_entries = get_objects(
            driver,
            By.XPATH,
            "//div[@id='education']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each education entry
        for entry in education_entries:
            try:
                education_data = {}

                # Extract organization name
                try:
                    university = extract_element_text(
                        entry,
                        By.XPATH,
                        ".//a[contains(@target, '_self')]//span[contains(@aria-hidden, 'true')]",
                    )
                    education_data["university"] = university
                except Exception as e:
                    print(f"Error extracting university: Not Available")
                    education_data["university"] = None

                # Extract degree and field of study
                try:
                    degree_field = extract_element_text(
                        entry,
                        By.XPATH,
                        ".//span[contains(@class, 't-14 t-normal') and not(contains(@aria-hidden, 'true'))]",
                    )
                    degree_field = degree_field.split("\n")[0].strip()
                    education_data["degree_field"] = degree_field
                except Exception as e:
                    print(f"Error extracting degree and field of study: Not Available")
                    education_data["degree_field"] = None

                # Extract graduation year
                try:
                    graduation_year = extract_element_text(
                        entry, By.XPATH, ".//span[@class='pvs-entity__caption-wrapper']"
                    )
                    education_data["graduation_year"] = graduation_year
                except Exception as e:
                    print(f"Error extracting graduation year: Not Available")
                    education_data["graduation_year"] = None

                # # Extract skills
                # try:
                #     skills = extract_element_text(
                #         entry,
                #         By.XPATH,
                #         ".//strong[contains(text(), 'skill')]"
                #     )
                #     if not skills:
                #         skills = None
                #     education_data["skills"] = skills or ""
                # except Exception as e:
                #     print(f"Error extracting skills: not available")
                #     education_data["skills"] = None

                educations.append(education_data)

            except Exception as e:
                print(f"Error extracting one education entry: Not Available")
    except Exception as e:
        print(f"Error extracting education information: Not Available")
    return educations

def extract_certificates(driver):
    certificates = []

    try:
        # Locate all certificate entries
        certificate_entries = get_objects(
            driver,
            By.XPATH,
            "//div[@id='licenses_and_certifications']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each certificate entry
        for entry in certificate_entries:
            try:
                certificate_data = {}

                # Extract certificate name
                try:
                    cert_name = extract_element_text(
                        entry,
                        By.XPATH,
                        ".//div[contains(@class, 'display-flex')]//span[contains(@aria-hidden, 'true')]",
                    )
                    certificate_data["cert_name"] = cert_name.strip()
                except Exception as e:
                    print(f"Error extracting certificate name: Not Available")
                    certificate_data["cert_name"] = None

                # Extract issuing organization
                try:
                    issuer = extract_element_text(
                        entry,
                        By.XPATH,
                        ".//span[@class='t-14 t-normal']//span[contains(@aria-hidden, 'true')]",
                    )
                    certificate_data["issuer"] = issuer.strip()
                except Exception as e:
                    print(f"Error extracting issuing organization: Not Available")
                    certificate_data["issuer"] = None

                # Extract issue date
                try:
                    issue_date = extract_element_text(
                        entry,
                        By.XPATH,
                        ".//span[@class='pvs-entity__caption-wrapper' and contains(@aria-hidden, 'true')]",
                    )
                    certificate_data["issue_date"] = issue_date.strip()
                except Exception as e:
                    print(f"Error extracting issue date: Not Available")
                    certificate_data["issue_date"] = None

                # Extract credential URL (if available)
                try:
                    credential_url = extract_element_attribute(
                        entry,
                        By.XPATH,
                        "//div[contains(@class, 'display-flex flex-column')]//div[contains(@class, 'pvs-entity__sub-components')]//a[contains(@class, 'artdeco-button')]",
                        "href",
                    )
                    certificate_data["credential_url"] = credential_url
                except Exception as e:
                    print(f"Error extracting credential URL: Not Available")
                    certificate_data["credential_url"] = None

                certificates.append(certificate_data)

            except Exception as e:
                print(f"Error extracting one certificate entry: Not Available")
    except Exception as e:
        print(f"Error: certificates not available")
    return certificates

def extract_project(driver):
    projects = []

    try:
        projects_element = get_objects(
            driver,
            By.XPATH,
            "//div[@id='projects']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each project and extract details
        for project in projects_element:
            project_data = {}

            # Extract project title
            try:
                project_title = extract_element_text(
                    project,
                    By.XPATH,
                    ".//div[contains(@class, 'mr1 t-bold')]/span[@aria-hidden='true']",
                )
                project_data["project_title"] = project_title
            except:
                project_data["dates"] = None

            # Extract associated dates (if available)
            try:
                dates = extract_element_text(
                    project, By.XPATH, ".//span[@class='t-14 t-normal']"
                ).split("\n")[0]
                project_data["dates"] = dates
            except:
                project_data["dates"] = None

            # Extract associated organization (if available)
            try:
                organization = extract_element_text(
                    project,
                    By.XPATH,
                    ".//span[contains(text(), 'Associated with')]/following-sibling::span",
                )
                project_data["organization"] = organization
            except:
                project_data["organization"] = None

            # Extract description (if available)
            try:
                description = extract_element_text(
                    project,
                    By.XPATH,
                    ".//li[contains(@class, 'pvs-list__item--with-top-padding')]//div[contains(@class, 'full-width t-14')]//div[contains(@class, 'full-width')]//span[@aria-hidden='true']",
                )
                project_data["description"] = description
            except:
                project_data["description"] = None

            # Extract related link (if available)
            try:
                link = extract_element_attribute(
                    project,
                    By.XPATH,
                    ".//a[@class='optional-action-target-wrapper']",
                    "href",
                )
                project_data["link"] = link
            except:
                project_data["link"] = None

            projects.append(project_data)
    except:
        print(f"Error: projects not available")
    return projects

def extract_volunteering(driver):
    volunteers = []

    try:
        # Locate all volunteering entries (multiple li elements within the ul)
        volunteering_entries = get_objects(
            driver,
            By.XPATH,
            "//div[@id='volunteering_experience']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each volunteering entry
        for entry in volunteering_entries:
            volunteer_data = {}

            # Try to extract the role or title
            try:
                role = extract_element_text(
                    entry, By.XPATH, ".//div[contains(@class,'t-bold')]"
                )
                volunteer_data["role"] = role
            except Exception as e:
                print(f"Error extracting role: Not Available")
                volunteer_data["role"] = None

            # Try to extract the organization name
            try:
                organization = extract_element_text(
                    entry, By.XPATH, ".//span[contains(@class, 't-14 t-normal')]"
                ).split("\n")[0]
                volunteer_data["organization"] = organization
            except Exception as e:
                print(f"Error extracting organization: Not Available")
                volunteer_data["organization"] = None

            # Try to extract the duration
            try:
                duration = extract_element_text(
                    entry, By.XPATH, ".//span[@class='pvs-entity__caption-wrapper']"
                )
                volunteer_data["duration"] = duration
            except Exception as e:
                print(f"Error extracting duration: Not Available")
                volunteer_data["duration"] = None

            # Try to extract the cause (if available)
            try:
                cause = extract_many_element_text(
                    entry, By.XPATH, ".//span[contains(@class, 't-black--light')]"
                )
                volunteer_data["cause"] = cause[1].split("\n")[0]
            except Exception as e:
                print(f"Error extracting cause: Not available")
                volunteer_data["cause"] = None

            # Try to extract the description (if available)
            try:
                description = extract_element_text(
                    entry, By.XPATH, ".//div[contains(@class, 'inline-show-more-text')]"
                )
                volunteer_data["description"] = description
            except Exception as e:
                print(f"Error extracting volunteer description: Not Available")
                volunteer_data["description"] = None

            # Add the collected data for the current entry
            volunteers.append(volunteer_data)
    except Exception as e:
        print(f"Error: volunteering not available")

    return volunteers

def extract_skill(driver):
    skills = []
    try:
        skills_element = get_objects(
            driver,
            By.XPATH,
            "//div[@id='skills']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each skill and extract details
        for skill in skills_element:
            skill_data = {}
            # Extract skill title
            skill_title = extract_element_text(
                skill,
                By.XPATH,
                ".//div[contains(@class, 'hoverable-link-text')]/span[@aria-hidden='true']",
            )
            skill_data["title"] = skill_title

            # Extract endorsements_by, if available
            try:
                endorsements_by = extract_many_element_text(
                    skill,
                    By.XPATH,
                    ".//div[contains(@class, 'mv1')]//div[@class='display-flex ']//div[contains(@class, 'inline-show-more-text--is-collapsed')]",
                )
                skill_data["endorsements_by"] = endorsements_by
            except:
                skill_data["endorsements_by"] = None

            # Extract endorsements, if available
            try:
                endorsements = extract_element_text(
                    skill,
                    By.XPATH,
                    ".//span[contains(@aria-hidden, 'true') and contains(text(), 'endorsements')]",
                )
                skill_data["endorsements"] = endorsements.split()[0]
            except:
                skill_data["endorsements"] = None

            skills.append(skill_data)

    except Exception as e:
        print(f"Error extracting skills information: Not Available")
    return skills

def extract_honor(driver):
    honors = []
    try:
        honors_xpath = "//div[@id='honors_and_awards']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]"

        # Find all honors and awards elements
        honors_elements = get_objects(driver, By.XPATH, honors_xpath)

        # Iterate through each honor/award item and extract details
        for honor in honors_elements:
            # Extract the title of the honor
            title = extract_element_text(
                honor, By.XPATH, ".//div[contains(@class, 't-bold')]/span"
            )

            # Extract the issuer and date
            issuer_date = (
                extract_element_text(
                    honor, By.XPATH, ".//span[contains(@class, 't-14 t-normal')]"
                )
                .split("\n")[0]
                .split("·")
            )

            # Extract image or media link if present
            try:
                media_url = extract_element_attribute(
                    honor,
                    By.XPATH,
                    ".//a[contains(@class, 'optional-action-target-wrapper')]",
                    "href",
                )
            except:
                media_url = None

            education_data = {
                "title": title,
                "issuing_institution": issuer_date[0].strip(),
                "issued_date": issuer_date[1].strip(),
                "media_url": media_url,
            }
            honors.append(education_data)

    except Exception as e:
        print(f"Error extracting honors information: Not Available")
    return honors

def extract_organizations(driver):
    organizations = []
    try:
        organization_entries = get_objects(
            driver,
            By.XPATH,
            "//div[@id='organizations']/ancestor::section//ul/li[contains(@class, 'artdeco-list__item')]",
        )

        # Loop through each organization entry
        for entry in organization_entries:
            organization_data = {}

            # Try to extract the organization name
            try:
                organization_name = extract_element_text(
                    entry, By.XPATH, ".//div[contains(@class,'t-bold')]"
                )
                organization_data["organization_name"] = organization_name
            except Exception as e:
                print(f"Error extracting organization name: Not Available")
                organization_data["organization_name"] = None

            # Try to extract the role or title and duration
            try:
                role_duration = extract_element_text(
                    entry, By.XPATH, ".//span[contains(@class, 't-14 t-normal')]"
                ).split("·")

                # Separate role and duration
                role = role_duration[0].strip() if len(role_duration) > 0 else None
                duration = role_duration[1].strip() if len(role_duration) > 1 else None
                organization_data["role"] = role
                organization_data["duration"] = duration
            except Exception as e:
                print(f"Error extracting role or duration: Not Available")
                organization_data["role"] = None
                organization_data["duration"] = None

            # Try to extract the description or other additional information
            try:
                description = extract_element_text(
                    entry,
                    By.XPATH,
                    ".//li[contains(@class, 'pvs-list__item--with-top-padding')]//div[contains(@class, 't-14 t-normal t-black')]",
                )
                organization_data["description"] = description
            except Exception as e:
                print(f"Error extracting description: Not Available")
                organization_data["description"] = None

            # Append the collected data to the organizations list
            organizations.append(organization_data)

    except Exception as e:
        print(f"Error extracting organizations information: Not Available")
    return organizations

def extract_more_profiles(driver):
    """Extract 'More profiles for you' section URLs."""
    try:
        # Locate the section with profiles similar to the current one
        profile_urls = get_object(
            driver, By.XPATH, "//a[@data-field='browsemap_card_click']"
        )

        # If URLs are found, extract them
        if profile_urls:
            url = profile_urls.get_attribute("href")
            parsed_url = urlparse(url)
            url = urlunparse(parsed_url._replace(query=""))  # Remove the query part
            return url
        else:
            raise NoSuchElementException("No 'More profiles for you' section found.")

    except NoSuchElementException as e:
        print(f"Error: Not Available. No profiles were found.")
        return []

    except Exception as e:
        print(f"An unexpected error occurred: Not Available")
        return []




def extract_profile(driver,profile_url):
    try:
        driver.get(profile_url)
    except Exception as e:
        print(f"Error opening profile: {profile_url} , Error: {e}")
        return None

    scroll_and_load(driver)

    filename = extract_and_decode_username(profile_url)

    profile_data = {}
    profile_data["filename"] = filename
    profile_data["profile_url"] = profile_url
    profile_data["intro"] = extract_intro(driver)
    profile_data["about"] = extract_about(driver)
    profile_data['experience'] = extract_experience(driver)
    profile_data['education'] = extract_education(driver)
    profile_data['certificate'] = extract_certificates(driver)
    profile_data['projects'] = extract_project(driver)
    profile_data["volunteering"] = extract_volunteering(driver)
    profile_data['skills'] = extract_skill(driver)
    profile_data['honor'] = extract_honor(driver)
    profile_data['organizations'] = extract_organizations(driver)

    return profile_data


import concurrent.futures

def extract_profile_thread_pool(driver, profile_url):
    try:
        driver.get(profile_url)
    except Exception as e:
        print(f"Error opening profile: {profile_url}, Error: {e}")
        return None

    scroll_and_load(driver)  # Assuming this method is safe to call in parallel

    filename = extract_and_decode_username(profile_url)

    profile_data = {}
    profile_data["filename"] = filename
    profile_data["profile_url"] = profile_url

    # Using ThreadPoolExecutor for concurrent extraction of data
    with concurrent.futures.ThreadPoolExecutor() as executor:
        futures = {
            executor.submit(extract_intro, driver): "intro",
            executor.submit(extract_about, driver): "about",
            executor.submit(extract_experience, driver): "experience",
            executor.submit(extract_education, driver): "education",
            executor.submit(extract_certificates, driver): "certificate",
            executor.submit(extract_project, driver): "projects",
            executor.submit(extract_volunteering, driver): "volunteering",
            executor.submit(extract_skill, driver): "skills",
            executor.submit(extract_honor, driver): "honor",
            executor.submit(extract_organizations, driver): "organizations"
        }

        for future in concurrent.futures.as_completed(futures):
            key = futures[future]
            try:
                profile_data[key] = future.result()
            except Exception as e:
                print(f"Error extracting {key} for {profile_url}: {e}")
                profile_data[key] = None

    return profile_data



import os.path

from colorama import Fore, Style
from linkedin_cat.message import LinkedinMessage
from linkedin_cat.profile import extract_profile,extract_profile_thread_pool
from linkedin_cat.helper import save_to_json,extract_and_decode_username
from urllib.parse import urlencode,unquote
from bs4 import BeautifulSoup

class LinkedinSearch(LinkedinMessage):
    """
    Encapsulates the functionality related to interacting with
    LinkedIn using Selenium.
    """
    def __int__(self,linkedin_cookies_json:str,headless = False):
        super().__init__(linkedin_cookies_json,headless)

    def extract_username_from_linkedin_url(self,linkedin_url):
        """
        从 LinkedIn 个人资料 URL 中提取用户名并将其解码为汉字。

        参数:
            url (str): LinkedIn 个人资料的 URL。

        返回:
            str: 解码后的用户名。
        """
        # 去除结尾的 "/"
        linkedin_url = linkedin_url.rstrip('/')

        # 提取用户名部分
        username_encoded = linkedin_url.split('/')[-1]

        # 解码为汉字
        username = unquote(username_encoded)

        return username

    def generate_linkedin_search_url(self, keywords, company=None, title=None,school=None,
                                     first_name=None, last_name=None, origin="SWITCH_SEARCH_VERTICAL",sid=None):
        base_url = "https://www.linkedin.com/search/results/people/"
        """
        Generates a LinkedIn search URL based on the provided parameters.

        Args:
            base_url (str): The base LinkedIn search URL.
            keywords (str): Required search keywords.
            company (str, optional): Company name to filter results.
            first_name (str, optional): First name of the person to filter results.
            last_name (str, optional): Last name of the person to filter results.
            origin (str, optional): Origin of the search (e.g., 'FACETED_SEARCH').
            school (str, optional): School name or text to filter results.
            title (str, optional): Job title to filter results.
            sid (str, optional): Session ID or other identifier for the search context.

        Returns:
            str: A LinkedIn search URL with all applicable parameters.
        """
        # Ensure 'keywords' parameter is provided
        if not keywords:
            raise ValueError("The 'keywords' parameter is required.")

        # Initialize parameters with the required 'keywords'
        params = {
            'keywords': keywords
        }

        # Add optional parameters if they are provided
        if company:
            params['company'] = company
        if title:
            params['titleFreeText'] = title

        if last_name:
            params['lastName'] = last_name
        if first_name:
            params['firstName'] = first_name
        if school:
            params['schoolFreetext'] = school
        if origin:
            params['origin'] = origin
        if sid:
            params['sid'] = sid

        # Encode parameters to URL query format and append to the base URL
        search_url = f"{base_url}?{urlencode(params)}"
        return search_url


    def parse_linkedin_results(self,html_text):
        """
        Parses LinkedIn search results HTML to extract structured information from each profile.

        Parameters:
        - html_text: str, HTML content of the LinkedIn search results page.

        Returns:
        - results: list of dictionaries, each containing information of a LinkedIn profile (image, name, position, location, etc.).
        """
        soup = BeautifulSoup(html_text, 'html.parser')

        # Initialize an empty list to store all results
        results = []

        # Find all list items with class `reusable-search__result-container`
        profiles = soup.find_all('li', class_='reusable-search__result-container')

        for profile in profiles:
            # Initialize a dictionary to hold profile data
            profile_data = {}



            # Extract profile name and LinkedIn profile link
            name_tag = profile.find('span', {'aria-hidden': 'true'})
            profile_data['name'] = name_tag.get_text(strip=True) if name_tag else None

            # Extract company, location, and title information if available
            company_tag = profile.find('div', class_='entity-result__primary-subtitle')
            profile_data['title'] = company_tag.get_text(strip=True) if company_tag else None

            location_tag = profile.find('div', class_='entity-result__secondary-subtitle')
            profile_data['location'] = location_tag.get_text(strip=True) if location_tag else None


            title_tag = profile.find('p', class_='entity-result__summary')
            profile_data['introduction'] = title_tag.get_text() if title_tag else None

            link_tag = profile.find("span", class_="entity-result__title-text t-16")
            profile_data['linkedin_url'] = profile.find("span", class_="entity-result__title-text t-16").find("a")["href"] if link_tag else None

            # Extract profile image URL
            img_tag = profile.find('img', class_='presence-entity__image')
            if img_tag and img_tag.has_attr('src'):
                profile_data['image_url'] = img_tag['src']
            else:
                profile_data['image_url'] = None

            # Add profile data to results list
            results.append(profile_data)

        return results

    def open_linkedin_url(self,url):
        try:
            # Wait for a short duration
            self.short_wait()

            # Get the URL
            print(Fore.GREEN + f"Opening Linkedin URL: {url}" + Style.RESET_ALL)
            self.driver.get(url)

            # Wait for a medium duration
            self.medium_wait()

            # Scroll to the middle of the page
            print(Fore.GREEN + "Scrolling to the middle of the page" + Style.RESET_ALL)
            self.scroll_to_middle()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to a random position
            print(Fore.GREEN + "Scrolling to a random position" + Style.RESET_ALL)
            self.scroll_to_random()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to the bottom of the page
            print(Fore.GREEN + "Scrolling to the bottom of the page" + Style.RESET_ALL)
            self.scroll_to_bottom()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to the top of the page
            print(Fore.GREEN + "Scrolling to the top of the page" + Style.RESET_ALL)
            self.scroll_to_top()

            # Wait for a medium duration
            self.short_wait()
        except Exception as e:
            # If an exception occurs, print an error message
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)

        finally:
            return self.driver.page_source

    def search_keywords(self,keywords):
        try:
            # Wait for a short duration
            self.short_wait()

            url = self.generate_linkedin_search_url(keywords)

            html = self.open_linkedin_url(url)

            results = self.parse_linkedin_results(html)

            # Wait for a medium duration
            self.medium_wait()

            return results
        except Exception as e:
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)

    def search_linkedin_profile(self,url,save_folder='./linkedin',thread_pool=True):
        # if folder does not exist,create it
        if not os.path.exists(save_folder):
            os.mkdir(save_folder)

        filename = extract_and_decode_username(url)
        if os.path.exists(os.path.join(save_folder,f"{filename}.json")):
            print(Fore.YELLOW + f"Profile {filename} already exists" + Style.RESET_ALL)
            return

        try:
            print(Fore.GREEN + "Opening LinkedIn URL" + Style.RESET_ALL)
            profile_data =  extract_profile_thread_pool(self.driver,url) if thread_pool else extract_profile(self.driver,url)
            if not profile_data:
                print(Fore.RED + "Could not extract profile" + Style.RESET_ALL)

            else:
                print(Fore.GREEN + "Saving profile data to JSON" + Style.RESET_ALL)
                file_path = os.path.join(save_folder,f"{profile_data['filename']}.json")
                save_to_json(profile_data,file_path)

        except Exception as e:
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)

    def search_linkedin_profile_list(self,url_list,save_folder='./linkedin'):
        for url in url_list:
            try:
                self.search_linkedin_profile(url,save_folder)
            except Exception as e:
                print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)


    def close_driver(self):
        self.driver.quit()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_driver()





import time
import random
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.by import By
from colorama import Fore, Style
from typing import List
from linkedin_cat.base import LinkedinBase

class LinkedinMessage(LinkedinBase):
    """
    Encapsulates the functionality related to interacting with
    LinkedIn using Selenium.
    """
    def __int__(self,linkedin_cookies_json:str,headless = False):
        super().__init__(linkedin_cookies_json,headless)

    def open_linkedin_url(self,url):
        try:
            # Wait for a short duration
            self.short_wait()

            # Get the URL
            print(Fore.GREEN + f"Opening Linkedin URL: {url}" + Style.RESET_ALL)
            self.driver.get(url)

            # Wait for a medium duration
            self.medium_wait()

            # Scroll to the middle of the page
            print(Fore.GREEN + "Scrolling to the middle of the page" + Style.RESET_ALL)
            self.scroll_to_middle()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to a random position
            print(Fore.GREEN + "Scrolling to a random position" + Style.RESET_ALL)
            self.scroll_to_random()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to the bottom of the page
            print(Fore.GREEN + "Scrolling to the bottom of the page" + Style.RESET_ALL)
            self.scroll_to_bottom()

            # Wait for a short duration
            self.medium_wait()

            # Scroll to the top of the page
            print(Fore.GREEN + "Scrolling to the top of the page" + Style.RESET_ALL)
            self.scroll_to_top()

            # Wait for a medium duration
            self.short_wait()
        except Exception as e:
            # If an exception occurs, print an error message
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)
    def send_connection_request(self, message):
        """
        Sends a connection request to a LinkedIn profile.

        Returns: True if connection request was successful, False otherwise.

        Parameter message: The connection message being sent.
        Precondition: message must be a String where len(message) <= 300.
        """
        try:
            print(Fore.GREEN + "Locating the 'Connect' button" + Style.RESET_ALL)
            connect_button = self.driver.find_element(By.XPATH, "(//button[contains(@aria-label, 'Invite') "
                                                                "and contains(@class, 'pvs-profile-actions__action')])")
            connect_button.click()
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Connect' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating the 'Add a note' button" + Style.RESET_ALL)
            add_note_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Add a note']")
            add_note_button.click()
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Add a note' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating the message entry field" + Style.RESET_ALL)
            message_entry = self.driver.find_element(By.XPATH, "//textarea[@name='message']")
            message_entry.send_keys(message)
            self.medium_wait()
            print(Fore.GREEN + "Typed the message" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating the 'Send now' button" + Style.RESET_ALL)
            # send_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Send now']")
            send_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Send invitation']")
            send_button.click()
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Send now' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Verifying if connection request is sent" + Style.RESET_ALL)
            self.driver.find_element(By.XPATH, "//button[contains(@aria-label, 'Pending') "
                                               "and contains(@class, 'pvs-profile-actions__action') "
                                               "and .//span[contains(@class, 'artdeco-button__text') "
                                               "and text()='Pending']]")
            print(Fore.GREEN + "Connection request sent" + Style.RESET_ALL)
            return True

        except Exception as e:
            print(Fore.RED + f"Failed to send connection request, Error: {e}" + Style.RESET_ALL)
            return False

    def more_then_connect(self, message):
        """
        Sends a conenction request to a LinkedIn profile when the blue
        connection button isn't visible.

        Returns: True if the connection request was successful, False otherwise.

        Parameter message: The connection message being sent.
        Precondition: message must be a String where len(message) <= 300.
        """
        try:
            print(Fore.GREEN + "Locating and clicking the 'More' button" + Style.RESET_ALL)
            more_button = self.driver.find_element(By.XPATH, "(//button[contains(@class, 'artdeco-dropdown__trigger') "
                                                             "and contains(@class, 'artdeco-dropdown__trigger--placement-"
                                                             "bottom') and contains(@class, 'ember-view') and contains(@class,"
                                                             " 'pvs-profile-actions__action') and contains(@class, "
                                                             "'artdeco-button') and contains(@class, "
                                                             "'artdeco-button--secondary') and contains(@class, "
                                                             "'artdeco-button--muted') and contains(@class,"
                                                             " 'artdeco-button--2') and @aria-label='More actions'])[2]")
            more_button.click()
            self.driver.execute_script("window.scrollBy(0, 300)")
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'More' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating and clicking the 'Connect' button" + Style.RESET_ALL)
            connect_button = self.driver.find_element(By.XPATH, "(//div[contains(@aria-label, 'Invite') and "
                                                                "contains(@aria-label, 'to connect')])")
            self.driver.execute_script("arguments[0].click();", connect_button)
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Connect' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating and clicking the 'Add a note' button" + Style.RESET_ALL)
            add_note_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Add a note']")
            add_note_button.click()
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Add a note' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating the message entry field" + Style.RESET_ALL)
            message_entry = self.driver.find_element(By.XPATH, "//textarea[@name='message']")
            message_entry.send_keys(message)
            self.medium_wait()
            print(Fore.GREEN + "Typed the message" + Style.RESET_ALL)

            print(Fore.GREEN + "Locating and clicking the 'Send now' button" + Style.RESET_ALL)
            # send_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Send now']")
            send_button = self.driver.find_element(By.XPATH, "//button[@aria-label='Send invitation']")
            send_button.click()
            self.medium_wait()
            print(Fore.GREEN + "Clicked on the 'Send now' button" + Style.RESET_ALL)

            print(Fore.GREEN + "Verifying if connection request is sent" + Style.RESET_ALL)
            self.driver.find_element('xpath', "//div[contains(@aria-label, 'Pending') and "
                                              "contains(@class, 'artdeco-dropdown__item')]")
            print(Fore.GREEN + "Connection request sent" + Style.RESET_ALL)
            return True

        except WebDriverException:
            print(Fore.RED + "Failed to send connection request" + Style.RESET_ALL)
            return False

    def is_friend(self):
        try:
            print(Fore.GREEN + "Checking friend status" + Style.RESET_ALL)
            friend_status = self.driver.find_element(By.XPATH,
                                                     "//span[contains(@class, 'distance-badge')]/span[contains(@class, 'dist-value')]")
            if friend_status.text.strip() == "1st":
                print(Fore.GREEN + "Friend status: True" + Style.RESET_ALL)
                return True
            else:
                print(Fore.GREEN + "Friend status: False" + Style.RESET_ALL)
                return False
        except NoSuchElementException:
            print(Fore.RED + "Friend status: Element not found" + Style.RESET_ALL)
            return False
        finally:
            self.short_wait()

    def is_pending(self):
        try:
            print(Fore.GREEN + "Checking pending status" + Style.RESET_ALL)
            pending_status = self.driver.find_element(By.XPATH, ("//button[contains(@aria-label, 'Pending') and "
                                                                 "contains(@class, 'pvs-profile-actions__action') and .//span[contains(@class, "
                                                                 "'artdeco-button__text') and text()='Pending']]"))
            print(Fore.GREEN + "Pending status: True" + Style.RESET_ALL)
            return True
        except NoSuchElementException:
            print(Fore.GREEN + "Pending status: False" + Style.RESET_ALL)
            return False
        finally:
            self.short_wait()

    def has_connect_button(self):
        """
        Checks if LinkedIn gives the user the option to connect with the current profile.

        Returns: True if the profile has the conenct button, False otherwise.
        """
        try:
            connect = self.driver.find_element(By.XPATH,
                                               "(//span[@class='artdeco-button__text'])[6]")
            connect_following = self.driver.find_element(By.XPATH,
                                                         "(//span[@class='artdeco-button__text'])[8]")
            if connect.text == "Connect" or connect_following.text == "Connect":
                return True
            else:
                return False
        except NoSuchElementException:
            return False

    def has_hidden_connect_button(self):
        """
        Checks if the user can send a request after clicking more.

        Returns True if the profile has a hidden connect button, False otherwise.
        """
        try:
            self.driver.find_element(By.XPATH,
                                     ("(//div[contains(@class, 'artdeco-dropdown__item') and "
                                      "contains(@class, 'artdeco-dropdown__item--is-dropdown')"
                                      " and contains(@class, 'ember-view') and contains(@class,"
                                      " 'full-width') and contains(@class, 'display-flex') and"
                                      " contains(@class, 'align-items-center')]/"
                                      "span[text()='Connect'])[1]"))
            return True
        except NoSuchElementException:
            return False

    def generate_message(self,message:str):
        """
        Extracts the first name from the LinkedIn profile page,
        then assembles the personalized message to send. Allows the
        user the put [FULL NAME] and [FIRST NAME] in their message to instruct the
        program to automatically insert the profile's name.

        Returns: A String representing the personalized connection message.
        """
        # full_name = self.driver.find_element(By.XPATH,
        #                                      ('(//h1[@class="text-heading-xlarge inline t-24'
        #                                       ' v-align-middle break-words"])[1]')).text

        full_name = self.driver.find_element(By.XPATH, '//h1[contains(@class, "inline t-24 v-align-middle break-words")]').text

        name_list = full_name.split(" ")
        first = name_list[0]
        if first.lower() not in ['dr.', 'mr.' 'mrs.',
                                 'ms.', 'dr', 'mr', 'mrs' 'ms']:
            first_name = name_list[0]
        else:
            first_name = name_list[1]

        msg = message.replace('FULLNAME', full_name)
        msg = msg.replace('FIRSTNAME', first_name)
        return msg.strip()

    def send_msg_to_friend(self,message:str):
        try:
            msg_button = self.driver.find_element(By.XPATH,("(//button[contains(@aria-label,'Message') and contains(@class, 'pvs-profile-actions__action')])"))
            msg_button.click()
            time.sleep(random.uniform(3, 5))

            msg_box = self.driver.find_element(By.XPATH, "//div[@contenteditable='true']/p")

            msg_box.click()
            last_msg = self.find_last_message()

            if last_msg == message:
                print(Fore.RED + f'Message already sent:{last_msg}' +  Style.RESET_ALL)
                return

            # 清空消息输入框
            from selenium.webdriver.common.keys import Keys
            msg_box.send_keys(Keys.CONTROL + "a")  # Windows/Linux系统可以用CTRL+A；如果是macOS，使用Keys.COMMAND
            msg_box.send_keys(Keys.DELETE)

            time.sleep(random.uniform(3, 5))

            # print('box count', self.get_msg_box_count())
            # if self.get_msg_box_count() != 1:
            #     print(Fore.RED + f'Message box count is not 1' +  Style.RESET_ALL)
            #     self.close_msg_box()
            #     print(Fore.RED + f'Closing boxes' +  Style.RESET_ALL)

            # 重新定位 msg_box
            msg_box = self.driver.find_element(By.XPATH, "//div[@contenteditable='true']/p")
            msg_box.send_keys(message)

            # send_button =  self.driver.find_element(By.XPATH, ("(//button[contains(@class,'msg-form__send-button') and contains(@type, 'submit')])"))

            # 等待发送按钮可点击
            from selenium.webdriver.support.ui import WebDriverWait
            from selenium.webdriver.support import expected_conditions as EC
            send_button = WebDriverWait(self.driver, 10).until(
                EC.element_to_be_clickable(
                    (By.XPATH, "(//button[contains(@class,'msg-form__send-button') and contains(@type, 'submit')])"))
            )

            time.sleep(random.uniform(1, 3))
            send_button.click()
            time.sleep(random.uniform(1, 3))
            print(Fore.BLUE + f'Message successfully sent:{message}'+ Style.RESET_ALL)
        except Exception as e:
            print(Fore.RED + f'Error sending message: {e}'+ Style.RESET_ALL)

    def send_single_request(self, url: str, message: str):
        try:
            print(Fore.BLACK + "="*30 + " Sending Request "+ "="*30 + Style.RESET_ALL)
            self.open_linkedin_url(url)

            msg = self.generate_message(message)
            print(Fore.BLUE + "Generating message:", msg + Style.RESET_ALL)

            while self.is_msg_box_exist():
                print(Fore.YELLOW + "Message box exists, Closing it now" + Style.RESET_ALL)
                self.close_msg_box()

            if self.is_friend():
                print(Fore.GREEN + "Sending message to friend" + Style.RESET_ALL)
                self.send_msg_to_friend(msg)
                return

            if self.is_pending():
                print(Fore.YELLOW + 'Connection request is pending:', url + Style.RESET_ALL)
                return

            if self.has_hidden_connect_button():
                print(Fore.GREEN + "Taking more than connect action" + Style.RESET_ALL)
                self.more_then_connect(msg)
            else:
                print(Fore.GREEN + "Sending connection request" + Style.RESET_ALL)
                self.send_connection_request(msg)
        except Exception as e:
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)
        finally:
            time.sleep(random.uniform(1, 3))

    def send_multi_request(self, url_list: List[str], message: str):
        try:
            for url in url_list:
                self.send_single_request(url,message)
        except Exception as e:
            print(Fore.RED + f'Error: {e}' + Style.RESET_ALL)
        finally:
            self.close_driver()

    def is_msg_box_exist(self):
        try:
            self.driver.find_element(By.XPATH,
                                                '(//button[contains(@class,"msg-overlay-bubble-header__control artdeco-button artdeco-button--circle artdeco-button--muted")])')
            return True
        except NoSuchElementException:
            return False

    def get_msg_box_count(self):
        try:
            elements = self.driver.find_elements(
                By.XPATH,
                '(//button[contains(@class,"msg-overlay-bubble-header__control artdeco-button artdeco-button--circle artdeco-button--muted")])'
            )
            count = len(elements)
            if count == 0:
                print("No elements found matching the specified XPath.")
            return count  # 返回找到的元素数量
        except NoSuchElementException as e:
            print(f"Error: No elements found. Exception details: {e}")
            return 0

    def close_msg_box(self):
        try:
            while True:
                self.medium_wait()
                if self.is_msg_box_exist():
                    print(Fore.GREEN + "Locating and clicking the 'Close' button" + Style.RESET_ALL)
                    close_button = self.driver.find_element(By.XPATH,
                                                            '(//button[contains(@class, "msg-overlay-bubble-header__control artdeco-button artdeco-button--circle artdeco-button--muted")])')
                    close_button.click()
                else:
                    break
                self.short_wait()
        except NoSuchElementException:
            print(Fore.RED + "Failed to close the message box" + Style.RESET_ALL)

    def find_last_message(self):
        try:
            messages = self.driver.find_elements(By.CSS_SELECTOR,'.msg-s-event__content')
            if messages:
                last_message = messages[-1].find_element(By.TAG_NAME, 'p').text
                return last_message.strip()
        except NoSuchElementException:
            return False

    def close_driver(self):
        self.driver.quit()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close_driver()